//import your libraries. 
import de.bezier.data.sql.*;

MySQL msql;
String[] DeweyNames = {"Computer Science & Information", "Philosophy & Psychology", "Religion", "Social Science", "Languages", "Science & Mathematics", "Technology and Applied Science", "Arts and Recreation", "Literature", "History, Geography & Biography" };  
int num=6; 
ArrayList transactions = null;
float[][] freq = new float[10][31];
String[] weekdays ={"M","T","W","R","F","S","Su","M"};  
boolean daydisplay = false; 
boolean grid = false; 
int total = 0; 
int curIdx = 0;
int maxCnt = 0;
int curKeyIdx = (int) 'a';
int date; 
int dew; 
int dkey = 1; 

int gridspacing = 20; 
float dataMin, dataMax;
float barWidth = 10;
float plotX1, plotY1;
float plotX2, plotY2;
float labelX, labelY;
float colormap; 
Integrator[] interpolators; 

int rowCount;
int columnCount;
int currentColumn = 0;

float[] tabLeft, tabRight; 
float tabTop, tabBottom; 
float tabPad = 10; 

int dayMin = 1, dayMax = 31;
int[] years;

int dayInterval = 1;
int volumeInterval = 500;
int volumeIntervalMinor = 250;

PFont font0 = createFont("HelveticaNeue-Light",13);
PFont font1 = createFont("HelveticaNeue-Light",10);
PFont font2 = createFont("HelveticaNeue-Light",22);

void setup(){
  size(900,600);
  background(0);
  frameRate(60);

  dataMin = 0;
  dataMax = 2500;
  plotX1 = 120; 
  plotX2 = width - 60;
  labelX = 40;
  plotY1 = 60;
  plotY2 = height - 80;
  labelY = height - 40;
  
    // connecting to the KRAKEN database
    msql = new MySQL( this, "kraken.mat.ucsb.edu", "spl0", "josh", "sneezes with wolves" ); //host, database, user, password
    if ( msql.connect() )
    { 
      String TheQuery="select floor(deweyClass/100) as dewey,day(cin),count(*) from inraw where deweyClass < 1000 and cin >= '2008-01-01' and cin <= '2008-01-31' group by day(cin), dewey order by dewey, day(cin);";
          println(TheQuery);
          msql.query( TheQuery);
          msql.next();
          while( msql.next() ){
              freq[msql.getInt(1)][msql.getInt(2)-1] = msql.getInt(3);
          }     
     }
    else
    {
      println(" connection failed !");
    }
  
  interpolators = new Integrator[31];
  for (int row = 0; row < 31; row++) {
    float initialValue = freq[num][row];
    total = total + (int)freq[num][row]; 
    interpolators[row] = new Integrator(initialValue);
    interpolators[row].attraction = 0.15;
  }
  println("KEY CONTROLS:");
  println("VIEW GRID \t\tg");
  println("VIEW DAYS \t\td");
  println("NEXT CATEGORY \t]");
  println("PREVIOUS CATEORY \t[");
  println("GRAPH MODES \t1-5");   
  smooth(); 
}

void draw(){
  background(255); 
  drawGrid(); 
  
  fill(128);
  rectMode(CORNERS);
  stroke(100);
  strokeWeight(1);
  rect(plotX1, plotY1, plotX2, plotY2);
  
  drawTitle();
  drawAxisLabels();
  drawVolumeLabels();
  drawDayLabels();
  
  for(int row=0; row < 31; row++){
    interpolators[row].update();
  }
  fill(255);
  //figure out what drawing mode you're in and draw the graph that way
  if(dkey == 1){ drawDataPoints();}
  if(dkey == 2){ drawDataArea();}
  if(dkey == 3){ drawDataLine();}
  if(dkey == 4){ drawDataCurve(); }
  if(dkey == 5){ drawDataBars(); }
  drawDataHighlight();
  textFont(font1); 
  fill(180); 
  textAlign(RIGHT);
  text("REZA ALI - 2009 - MAT 259 - INFORMATION VISUALIZATION", width-5, height-15);
  text("JOSH D - 2011 - REVISION", width-5, height-5);         
}

void drawGrid(){
  if(grid){
    for(int k = 0; k < width; k += gridspacing){
      stroke(128,80); 
      strokeWeight(.5);  
      line(0,k,width,k); 
      line(k,0,k,height); 
    }
  }
}

void drawDayLabels() {
  fill(100);
  //textSize(10);
  textAlign(CENTER, TOP);
  
  // Use thin, gray lines to draw the grid
  stroke(255,20);
  strokeWeight(1);

  for (int row = 0; row <= 31; row++) {
    if (row % dayInterval == 0) {
      float x = map(row, dayMin, dayMax, plotX1, plotX2);
      textFont(font0); 
      if (row == dayMin) {
          textAlign(LEFT, CENTER);                 // Align by the bottom
        } else if (row == dayMax) {
          textAlign(RIGHT, CENTER);            // Align by the top
        } else {
          textAlign(CENTER, CENTER);         // Center vertically
        }  
        if(row != 0){
          fill(100);
          text(row, x, plotY2 + 10);
          if(daydisplay){
            fill(170);
            text(weekdays[row%weekdays.length], x, plotY2 + 23);
          }
          line(x, plotY1, x, plotY2);
        }
    }
  }
}

void drawVolumeLabels() {
  fill(100);
  //textSize(10);
  textFont(font0); 
  stroke(128);
  strokeWeight(1);

  for (float v = dataMin; v <= dataMax; v += volumeIntervalMinor) {
    if (v % volumeIntervalMinor == 0) {     // If a tick mark
      float y = map(v, dataMin, dataMax, plotY2, plotY1);  
      if (v % volumeInterval == 0) {        // If a major tick mark
        if (v == dataMin) {
          textAlign(RIGHT);                 // Align by the bottom
        } else if (v == dataMax) {
          textAlign(RIGHT, TOP);            // Align by the top
        } else {
          textAlign(RIGHT, CENTER);         // Center vertically
        }
        text(floor(v), plotX1 - 10, y);
        line(plotX1 - 4, y, plotX1, y);     // Draw major tick
      } else {
        // Commented out, too distracting visually
        //line(plotX1 - 2, y, plotX1, y);   // Draw minor tick
      }
    }
  }
}


void drawTitle() {
  fill(70);
  //textSize(20);
  textAlign(LEFT);
  textFont(font2);
  text("Dewey Category: " + DeweyNames[num], plotX1, plotY1 - 10);
  textAlign(RIGHT);
  text("Total Items " + nf(total,6,0), plotX2, plotY1 - 10);  
}

void drawAxisLabels() {
  fill(100);
  //textSize(13);
  textLeading(15);
  textFont(font0);  
  textAlign(CENTER, CENTER);
  // Use \n (enter/linefeed) to break the text into separate lines
  text("# of Items\nChecked\nIn", labelX, (plotY1+plotY2)/2);
  textAlign(CENTER, CENTER);
  text("Days of January 2008", (plotX1+plotX2)/2, labelY);
}

void drawDataPoints() {
 for (int row = 0; row < 31; row++) {
    float value = interpolators[row].value;
    float x = map(row+1, dayMin, dayMax, plotX1, plotX2);
    float y = map(value, dataMin, dataMax, plotY2, plotY1);
    //println(value);
    strokeWeight(1.5); 
    line(x,plotY2,x,y);  
    stroke(255); 
    fill(128); 
    ellipse(x, y, 7, 7);
  }
}

void drawDataLine() {
  noFill(); 
  stroke(255);
  strokeWeight(1);
  beginShape();
  for (int row = 0; row < 31; row++) {
    float value = interpolators[row].value;
    float x = map(row+1, dayMin, dayMax, plotX1, plotX2);
    float y = map(value, dataMin, dataMax, plotY2, plotY1);
    vertex(x,y); 
  }
  endShape(); 
}

void drawDataArea() {
  noStroke(); 
  beginShape();
  for (int row = 0; row < 31; row++) {
    float value = interpolators[row].value;
    float x = map(row+1, dayMin, dayMax, plotX1, plotX2);
    float y = map(value, dataMin, dataMax, plotY2, plotY1);
    vertex(x,y); 
    }
  vertex(plotX2,plotY2);
  vertex(plotX1,plotY2);
  endShape(CLOSE); 
}

void drawDataBars() {
  noStroke(); 
  rectMode(CORNERS); 

 for (int row = 0; row < 31; row++) {
    float value = interpolators[row].value;
    float x = map(row+1, dayMin, dayMax, plotX1, plotX2);
    float y = map(value, dataMin, dataMax, plotY2, plotY1);
    rect(x-barWidth/2, y, x+barWidth/2,plotY2);
  }
}

void drawDataCurve() {
  noFill(); 
  strokeWeight(1);
  stroke(255); 
  beginShape();
  for (int row = 0; row < 31; row++) {
     float value = interpolators[row].value;
     float x = map(row+1, dayMin, dayMax, plotX1, plotX2);
     float y = map(value, dataMin, dataMax, plotY2, plotY1);
     curveVertex(x,y); 
     if((row == 0) || (row == 30)){
      curveVertex(x,y); 
     }
   }
  endShape(); 
}

void drawDataHighlight() {
  for (int row = 0; row < 31; row++) {
     float value = interpolators[row].value;
     float x = map(row+1, dayMin, dayMax, plotX1, plotX2);
     float y = map(value, dataMin, dataMax, plotY2, plotY1);
     if(dist(mouseX, mouseY, x, y) < 6){
        strokeWeight(7); 
        point(x,y);
        textFont(font0);  
        fill(60);  
        textAlign(CENTER); 
        text((int)value + " (" + nf((row+1),2,0) + ")", x, y-8);
      } 
   }
}

void keyPressed(){ 
  if(key == ']' || key == '['){
    if(key == ']'){ 
      num = num+1;
      if(num == 10){
        num = 9; 
      }
    }
    if(key =='['){
      num = num - 1; 
      if(num < 0){
        num = 0; 
      }
    }
      calTotal(); 
      setCurrent();
  }
  if(key == 'd'){
    daydisplay = !daydisplay; 
  } 
  if(key == 'g'){
    grid = !grid; 
  }
 if(key == '1'){
   dkey = 1; 
 }
 if(key == '2'){
   dkey = 2; 
 }
 if(key == '3'){
   dkey = 3; 
 }
 if(key == '4'){
   dkey = 4; 
 }
 if(key == '5'){
   dkey = 5; 
 }
} 

void setCurrent(){ 
  for(int row=0; row < 31; row++){
    interpolators[row].target(freq[num][row]);
  }
}

void calTotal(){
  total = 0; 
  for(int j = 0; j < freq[num].length; j++){
    total = total + (int)freq[num][j]; 
    }
}





